# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .forms_check_boxes_value_object_response_body import FormsCheckBoxesValueObjectResponseBody
from .forms_date_time_value_object_response_body import FormsDateTimeValueObjectResponseBody
from .forms_media_value_object_response_body import FormsMediaValueObjectResponseBody
from .forms_multiple_choice_value_object_response_body import FormsMultipleChoiceValueObjectResponseBody
from .forms_number_value_object_response_body import FormsNumberValueObjectResponseBody
from .forms_person_value_object_response_body import FormsPersonValueObjectResponseBody
from .forms_signature_value_object_response_body import FormsSignatureValueObjectResponseBody
from .forms_table_cell_object_response_body_type import FormsTableCellObjectResponseBodyType
from .forms_text_value_object_response_body import FormsTextValueObjectResponseBody


class FormsTableCellObjectResponseBody(UniversalBaseModel):
    """
    Defines a cell in a table row.
    """

    check_boxes_value: typing_extensions.Annotated[
        typing.Optional[FormsCheckBoxesValueObjectResponseBody],
        FieldMetadata(alias="checkBoxesValue"),
        pydantic.Field(alias="checkBoxesValue"),
    ] = None
    date_time_value: typing_extensions.Annotated[
        typing.Optional[FormsDateTimeValueObjectResponseBody],
        FieldMetadata(alias="dateTimeValue"),
        pydantic.Field(alias="dateTimeValue"),
    ] = None
    id: str = pydantic.Field()
    """
    Unique identifier for the cell.
    """

    media_value: typing_extensions.Annotated[
        typing.Optional[FormsMediaValueObjectResponseBody],
        FieldMetadata(alias="mediaValue"),
        pydantic.Field(alias="mediaValue"),
    ] = None
    multiple_choice_value: typing_extensions.Annotated[
        typing.Optional[FormsMultipleChoiceValueObjectResponseBody],
        FieldMetadata(alias="multipleChoiceValue"),
        pydantic.Field(alias="multipleChoiceValue"),
    ] = None
    number_value: typing_extensions.Annotated[
        typing.Optional[FormsNumberValueObjectResponseBody],
        FieldMetadata(alias="numberValue"),
        pydantic.Field(alias="numberValue"),
    ] = None
    person_value: typing_extensions.Annotated[
        typing.Optional[FormsPersonValueObjectResponseBody],
        FieldMetadata(alias="personValue"),
        pydantic.Field(alias="personValue"),
    ] = None
    signature_value: typing_extensions.Annotated[
        typing.Optional[FormsSignatureValueObjectResponseBody],
        FieldMetadata(alias="signatureValue"),
        pydantic.Field(alias="signatureValue"),
    ] = None
    text_value: typing_extensions.Annotated[
        typing.Optional[FormsTextValueObjectResponseBody],
        FieldMetadata(alias="textValue"),
        pydantic.Field(alias="textValue"),
    ] = None
    type: FormsTableCellObjectResponseBodyType = pydantic.Field()
    """
    Type of the cell field.  Valid values: `number`, `text`, `multiple_choice`, `check_boxes`, `datetime`, `signature`, `media`, `person`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

WorkOrderItemObjectResponseBodyType = typing.Union[
    typing.Literal[
        "DVIR",
        "FAULT",
        "FORM",
        "ISSUE",
        "ITEM_TYPE_UNSPECIFIED",
        "MAINTENANCE_PREDICTION_EVENT",
        "SCHEDULED_MAINTENANCE",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import enum


class SamsaraEnvironment(enum.Enum):
    PRODUCTION_API = "https://api.samsara.com"
    PRODUCTION_API_EU = "https://api.eu.samsara.com"



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.attribute_expanded_response import AttributeExpandedResponse
from ..types.create_attribute_request_entities import CreateAttributeRequestEntities
from ..types.get_attributes_by_entity_type_response import GetAttributesByEntityTypeResponse
from .raw_client import AsyncRawAttributesClient, RawAttributesClient
from .types.create_attribute_request_attribute_type import CreateAttributeRequestAttributeType
from .types.create_attribute_request_entity_type import CreateAttributeRequestEntityType
from .types.create_attribute_request_unit import CreateAttributeRequestUnit
from .types.delete_attributes_request_entity_type import DeleteAttributesRequestEntityType
from .types.get_attribute_request_entity_type import GetAttributeRequestEntityType
from .types.get_attributes_by_entity_type_request_entity_type import GetAttributesByEntityTypeRequestEntityType
from .types.update_attribute_request_attribute_type import UpdateAttributeRequestAttributeType
from .types.update_attribute_request_entity_type import UpdateAttributeRequestEntityType

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class AttributesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAttributesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawAttributesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAttributesClient
        """
        return self._raw_client

    def get_attributes_by_entity_type(
        self,
        *,
        entity_type: GetAttributesByEntityTypeRequestEntityType,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetAttributesByEntityTypeResponse:
        """
        Fetch all attributes in an organization associated with either drivers or assets.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Attributes** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        entity_type : GetAttributesByEntityTypeRequestEntityType
            Denotes the type of entity, driver or asset.

        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetAttributesByEntityTypeResponse
            All attributes in an organization for an entity type

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.attributes.get_attributes_by_entity_type(
            entity_type="driver",
        )
        """
        _response = self._raw_client.get_attributes_by_entity_type(
            entity_type=entity_type, limit=limit, after=after, request_options=request_options
        )
        return _response.data

    def create_attribute(
        self,
        *,
        attribute_type: CreateAttributeRequestAttributeType,
        entity_type: CreateAttributeRequestEntityType,
        name: str,
        entities: typing.Optional[typing.Sequence[CreateAttributeRequestEntities]] = OMIT,
        number_values: typing.Optional[typing.Sequence[float]] = OMIT,
        string_values: typing.Optional[typing.Sequence[str]] = OMIT,
        unit: typing.Optional[CreateAttributeRequestUnit] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AttributeExpandedResponse:
        """
        Creates a new attribute in the organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Attributes** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        attribute_type : CreateAttributeRequestAttributeType
            Denotes the data type of the attribute's values. Valid values: `single-select`, `multi-select`, `text`, `freeform-multi-select`, `number`, `date`.

        entity_type : CreateAttributeRequestEntityType
            Denotes the type of entity, driver or asset.

        name : str
            Name

        entities : typing.Optional[typing.Sequence[CreateAttributeRequestEntities]]
            Entities that will be applied to this attribute

        number_values : typing.Optional[typing.Sequence[float]]
            Number values that can be associated with this attribute

        string_values : typing.Optional[typing.Sequence[str]]
            String values that can be associated with this attribute

        unit : typing.Optional[CreateAttributeRequestUnit]
            Unit of the attribute (only for Number attributes).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AttributeExpandedResponse
            Newly created attribute object with ID.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.attributes.create_attribute(
            attribute_type="single-select",
            entity_type="driver",
            name="License Certifications",
        )
        """
        _response = self._raw_client.create_attribute(
            attribute_type=attribute_type,
            entity_type=entity_type,
            name=name,
            entities=entities,
            number_values=number_values,
            string_values=string_values,
            unit=unit,
            request_options=request_options,
        )
        return _response.data

    def get_attribute(
        self,
        id: str,
        *,
        entity_type: GetAttributeRequestEntityType,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AttributeExpandedResponse:
        """
        Fetch an attribute by id, including all of its applications.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Attributes** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            Samsara-provided UUID of the attribute.

        entity_type : GetAttributeRequestEntityType
            Denotes the type of entity, driver or asset.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AttributeExpandedResponse
            The attribute corresponding to request id.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.attributes.get_attribute(
            id="id",
            entity_type="driver",
        )
        """
        _response = self._raw_client.get_attribute(id, entity_type=entity_type, request_options=request_options)
        return _response.data

    def delete(
        self,
        id: str,
        *,
        entity_type: DeleteAttributesRequestEntityType,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Delete an attribute by id, including all of its applications.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Attributes** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            Samsara-provided UUID of the attribute.

        entity_type : DeleteAttributesRequestEntityType
            Denotes the type of entity, driver or asset.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.attributes.delete(
            id="id",
            entity_type="driver",
        )
        """
        _response = self._raw_client.delete(id, entity_type=entity_type, request_options=request_options)
        return _response.data

    def update_attribute(
        self,
        id: str,
        *,
        entity_type: UpdateAttributeRequestEntityType,
        attribute_type: typing.Optional[UpdateAttributeRequestAttributeType] = OMIT,
        entities: typing.Optional[typing.Sequence[CreateAttributeRequestEntities]] = OMIT,
        name: typing.Optional[str] = OMIT,
        number_values: typing.Optional[typing.Sequence[float]] = OMIT,
        string_values: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AttributeExpandedResponse:
        """
        Updates an attribute in the organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Attributes** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            Samsara-provided UUID of the attribute.

        entity_type : UpdateAttributeRequestEntityType
            Denotes the type of entity, driver or asset.

        attribute_type : typing.Optional[UpdateAttributeRequestAttributeType]
            Denotes the data type of the attribute's values. Valid values: `single-select`, `multi-select`, `text`, `freeform-multi-select`, `number`, `date`.

        entities : typing.Optional[typing.Sequence[CreateAttributeRequestEntities]]
            Entities that will be applied to this attribute

        name : typing.Optional[str]
            Name

        number_values : typing.Optional[typing.Sequence[float]]
            Number values that can be associated with this attribute

        string_values : typing.Optional[typing.Sequence[str]]
            String values that can be associated with this attribute

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AttributeExpandedResponse
            Newly created attribute object with ID.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.attributes.update_attribute(
            id="id",
            entity_type="driver",
        )
        """
        _response = self._raw_client.update_attribute(
            id,
            entity_type=entity_type,
            attribute_type=attribute_type,
            entities=entities,
            name=name,
            number_values=number_values,
            string_values=string_values,
            request_options=request_options,
        )
        return _response.data


class AsyncAttributesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAttributesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawAttributesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAttributesClient
        """
        return self._raw_client

    async def get_attributes_by_entity_type(
        self,
        *,
        entity_type: GetAttributesByEntityTypeRequestEntityType,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetAttributesByEntityTypeResponse:
        """
        Fetch all attributes in an organization associated with either drivers or assets.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Attributes** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        entity_type : GetAttributesByEntityTypeRequestEntityType
            Denotes the type of entity, driver or asset.

        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetAttributesByEntityTypeResponse
            All attributes in an organization for an entity type

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.attributes.get_attributes_by_entity_type(
                entity_type="driver",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_attributes_by_entity_type(
            entity_type=entity_type, limit=limit, after=after, request_options=request_options
        )
        return _response.data

    async def create_attribute(
        self,
        *,
        attribute_type: CreateAttributeRequestAttributeType,
        entity_type: CreateAttributeRequestEntityType,
        name: str,
        entities: typing.Optional[typing.Sequence[CreateAttributeRequestEntities]] = OMIT,
        number_values: typing.Optional[typing.Sequence[float]] = OMIT,
        string_values: typing.Optional[typing.Sequence[str]] = OMIT,
        unit: typing.Optional[CreateAttributeRequestUnit] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AttributeExpandedResponse:
        """
        Creates a new attribute in the organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Attributes** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        attribute_type : CreateAttributeRequestAttributeType
            Denotes the data type of the attribute's values. Valid values: `single-select`, `multi-select`, `text`, `freeform-multi-select`, `number`, `date`.

        entity_type : CreateAttributeRequestEntityType
            Denotes the type of entity, driver or asset.

        name : str
            Name

        entities : typing.Optional[typing.Sequence[CreateAttributeRequestEntities]]
            Entities that will be applied to this attribute

        number_values : typing.Optional[typing.Sequence[float]]
            Number values that can be associated with this attribute

        string_values : typing.Optional[typing.Sequence[str]]
            String values that can be associated with this attribute

        unit : typing.Optional[CreateAttributeRequestUnit]
            Unit of the attribute (only for Number attributes).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AttributeExpandedResponse
            Newly created attribute object with ID.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.attributes.create_attribute(
                attribute_type="single-select",
                entity_type="driver",
                name="License Certifications",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_attribute(
            attribute_type=attribute_type,
            entity_type=entity_type,
            name=name,
            entities=entities,
            number_values=number_values,
            string_values=string_values,
            unit=unit,
            request_options=request_options,
        )
        return _response.data

    async def get_attribute(
        self,
        id: str,
        *,
        entity_type: GetAttributeRequestEntityType,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AttributeExpandedResponse:
        """
        Fetch an attribute by id, including all of its applications.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Attributes** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            Samsara-provided UUID of the attribute.

        entity_type : GetAttributeRequestEntityType
            Denotes the type of entity, driver or asset.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AttributeExpandedResponse
            The attribute corresponding to request id.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.attributes.get_attribute(
                id="id",
                entity_type="driver",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_attribute(id, entity_type=entity_type, request_options=request_options)
        return _response.data

    async def delete(
        self,
        id: str,
        *,
        entity_type: DeleteAttributesRequestEntityType,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Delete an attribute by id, including all of its applications.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Attributes** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            Samsara-provided UUID of the attribute.

        entity_type : DeleteAttributesRequestEntityType
            Denotes the type of entity, driver or asset.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.attributes.delete(
                id="id",
                entity_type="driver",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, entity_type=entity_type, request_options=request_options)
        return _response.data

    async def update_attribute(
        self,
        id: str,
        *,
        entity_type: UpdateAttributeRequestEntityType,
        attribute_type: typing.Optional[UpdateAttributeRequestAttributeType] = OMIT,
        entities: typing.Optional[typing.Sequence[CreateAttributeRequestEntities]] = OMIT,
        name: typing.Optional[str] = OMIT,
        number_values: typing.Optional[typing.Sequence[float]] = OMIT,
        string_values: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AttributeExpandedResponse:
        """
        Updates an attribute in the organization.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Attributes** under the Setup & Administration category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            Samsara-provided UUID of the attribute.

        entity_type : UpdateAttributeRequestEntityType
            Denotes the type of entity, driver or asset.

        attribute_type : typing.Optional[UpdateAttributeRequestAttributeType]
            Denotes the data type of the attribute's values. Valid values: `single-select`, `multi-select`, `text`, `freeform-multi-select`, `number`, `date`.

        entities : typing.Optional[typing.Sequence[CreateAttributeRequestEntities]]
            Entities that will be applied to this attribute

        name : typing.Optional[str]
            Name

        number_values : typing.Optional[typing.Sequence[float]]
            Number values that can be associated with this attribute

        string_values : typing.Optional[typing.Sequence[str]]
            String values that can be associated with this attribute

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AttributeExpandedResponse
            Newly created attribute object with ID.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.attributes.update_attribute(
                id="id",
                entity_type="driver",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_attribute(
            id,
            entity_type=entity_type,
            attribute_type=attribute_type,
            entities=entities,
            name=name,
            number_values=number_values,
            string_values=string_values,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class OrganizationInfoCarrierSettings(UniversalBaseModel):
    """
    Carrier for a given organization.
    """

    carrier_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="carrierName"),
        pydantic.Field(alias="carrierName", description="Carrier for a given organization."),
    ] = None
    dot_number: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="dotNumber"),
        pydantic.Field(alias="dotNumber", description="Carrier US DOT Number for the organization."),
    ] = None
    main_office_address: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="mainOfficeAddress"),
        pydantic.Field(alias="mainOfficeAddress", description="Main office address for a given organization."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class QualificationTypeReferenceObjectResponseBody(UniversalBaseModel):
    """
    Qualification type (form template) reference.
    """

    id: str = pydantic.Field()
    """
    ID of the qualification type (template UUID)
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the qualification type
    """

    revision_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="revisionId"),
        pydantic.Field(
            alias="revisionId", description="ID of the qualification type revision (template revision UUID)"
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .workflow_incident_details_object_response_body import WorkflowIncidentDetailsObjectResponseBody


class WorkflowIncidentConditionObjectResponseBody(UniversalBaseModel):
    """
    Object representing the granular details of the condition. These details will vary depending on the condition.
    """

    description: str = pydantic.Field()
    """
    Descriptive name of the condition.
    """

    details: WorkflowIncidentDetailsObjectResponseBody
    trigger_id: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="triggerId"),
        pydantic.Field(
            alias="triggerId", description="Unique identifier describing the type of condition being represented."
        ),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class V1AssetLocationResponseItem(UniversalBaseModel):
    """
    Asset location details.
    """

    latitude: typing.Optional[float] = pydantic.Field(default=None)
    """
    The latitude of the location in degrees.
    """

    location: typing.Optional[str] = pydantic.Field(default=None)
    """
    The best effort (street,city,state) for the latitude and longitude.
    """

    longitude: typing.Optional[float] = pydantic.Field(default=None)
    """
    The longitude of the location in degrees.
    """

    speed_miles_per_hour: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="speedMilesPerHour"),
        pydantic.Field(
            alias="speedMilesPerHour",
            description="The speed calculated from GPS that the asset was traveling at in miles per hour.",
        ),
    ] = None
    time: typing.Optional[float] = pydantic.Field(default=None)
    """
    Time in Unix milliseconds since epoch when the asset was at the location.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .equipment_time import EquipmentTime


class EquipmentEngineTotalIdleTimeMinutes(UniversalBaseModel):
    """
    A time-series of engine total idle time minutes. (Beta)
    """

    time: EquipmentTime
    value: int = pydantic.Field()
    """
    Total time in minutes that the engine has been idling since the device was installed. This value is calculated based on the time the engine has been running while the vehicle is not in motion.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class EngineStatusResponseBody(UniversalBaseModel):
    """
    Equipment engine status.
    """

    running: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="Running"),
        pydantic.Field(alias="Running", description="Boolean value for whether engine is running or not."),
    ] = None
    datetime: typing.Optional[str] = pydantic.Field(default=None)
    """
    Date time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .goa_vehicle_tiny_response_response_body import GoaVehicleTinyResponseResponseBody
from .hos_eld_event_location_object_response_body import HosEldEventLocationObjectResponseBody
from .hos_eld_event_object_response_body_malfunction_diagnostic_code import (
    HosEldEventObjectResponseBodyMalfunctionDiagnosticCode,
)
from .hos_eld_event_remark_object_response_body import HosEldEventRemarkObjectResponseBody


class HosEldEventObjectResponseBody(UniversalBaseModel):
    """
    An event that refers to a discrete instance in time with various data elements. Depending on the type of event, not every field will be populated. For more information, see the ELD Mandate [section 3.1.2](https://www.ecfr.gov/cgi-bin/retrieveECFR?gp=1&ty=HTML&h=L&mc=true&=PART&n=pt49.5.395#ap49.5.395_138.a).
    """

    accumulated_vehicle_meters: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="accumulatedVehicleMeters"),
        pydantic.Field(
            alias="accumulatedVehicleMeters", description="The accumulated meters in the given ignition power on cycle."
        ),
    ] = None
    elapsed_engine_hours: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="elapsedEngineHours"),
        pydantic.Field(
            alias="elapsedEngineHours",
            description="The elapsed time in the engine's operation in the given ignition power on cycle.",
        ),
    ] = None
    eld_event_code: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="eldEventCode"),
        pydantic.Field(
            alias="eldEventCode",
            description="A dependent attribute on `eldEventType` that specifies the nature of the event, as defined in the ELD Mandate [section 7.20](https://www.ecfr.gov/cgi-bin/retrieveECFR?gp=1&ty=HTML&h=L&mc=true&=PART&n=pt49.5.395#ap49.5.395_138.a).  Valid values: `1`, `2`, `3`, `4`, `5`, `6`, `7`, `8`, `9`",
        ),
    ]
    eld_event_record_origin: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="eldEventRecordOrigin"),
        pydantic.Field(
            alias="eldEventRecordOrigin",
            description="An attribute for the event record indicating whether it is automatically recorded, or edited, entered or accepted by the driver, requested by another authenticated user, or assumed from unidentified driver profile, as defined in the ELD Mandate [section 7.22](https://www.ecfr.gov/cgi-bin/retrieveECFR?gp=1&ty=HTML&h=L&mc=true&=PART&n=pt49.5.395#ap49.5.395_138.a).  Valid values: `1`, `2`, `3`, `4`",
        ),
    ] = None
    eld_event_record_status: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="eldEventRecordStatus"),
        pydantic.Field(
            alias="eldEventRecordStatus",
            description="An attribute for the event record indicating whether an event is active or inactive and further, if inactive, whether it is due to a change or lack of confirmation by the driver or due to a driver's rejection of change request, as defined in the ELD Mandate [section 7.23](https://www.ecfr.gov/cgi-bin/retrieveECFR?gp=1&ty=HTML&h=L&mc=true&=PART&n=pt49.5.395#ap49.5.395_138.a).  Valid values: `1`, `2`, `3`, `4`",
        ),
    ] = None
    eld_event_type: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="eldEventType"),
        pydantic.Field(
            alias="eldEventType",
            description="An attribute specifying the type of ELD event, as defined in the ELD Mandate [section 7.25](https://www.ecfr.gov/cgi-bin/retrieveECFR?gp=1&ty=HTML&h=L&mc=true&=PART&n=pt49.5.395#ap49.5.395_138.a).  Valid values: `1`, `2`, `3`, `4`, `5`, `6`, `7`",
        ),
    ]
    location: typing.Optional[HosEldEventLocationObjectResponseBody] = None
    malfunction_diagnostic_code: typing_extensions.Annotated[
        typing.Optional[HosEldEventObjectResponseBodyMalfunctionDiagnosticCode],
        FieldMetadata(alias="malfunctionDiagnosticCode"),
        pydantic.Field(
            alias="malfunctionDiagnosticCode",
            description="A code that further specifies the underlying malfunction or data diagnostic event, as defined in the ELD Mandate [section 7.34](https://www.ecfr.gov/cgi-bin/retrieveECFR?gp=1&ty=HTML&h=L&mc=true&=PART&n=pt49.5.395#ap49.5.395_138.a).  Valid values: `P`, `E`, `T`, `L`, `R`, `S`, `O`, `1`, `2`, `3`, `4`, `5`, `6`",
        ),
    ] = None
    remark: typing.Optional[HosEldEventRemarkObjectResponseBody] = None
    time: str = pydantic.Field()
    """
    A time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).
    """

    total_engine_hours: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="totalEngineHours"),
        pydantic.Field(
            alias="totalEngineHours",
            description="The aggregated time of a vehicle's engine's operation since its inception.",
        ),
    ] = None
    total_vehicle_meters: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="totalVehicleMeters"),
        pydantic.Field(alias="totalVehicleMeters", description="The total meters recorded for the vehicle."),
    ] = None
    vehicle: typing.Optional[GoaVehicleTinyResponseResponseBody] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .speeding_settings_object_response_body_unit import SpeedingSettingsObjectResponseBodyUnit
from .speeding_severity_level_response_body import SpeedingSeverityLevelResponseBody


class SpeedingSettingsObjectResponseBody(UniversalBaseModel):
    """
    Enables custom speeding levels.
    """

    severity_levels: typing_extensions.Annotated[
        typing.Optional[typing.List[SpeedingSeverityLevelResponseBody]],
        FieldMetadata(alias="severityLevels"),
        pydantic.Field(alias="severityLevels", description="The speeding severity levels for an organization."),
    ] = None
    unit: typing.Optional[SpeedingSettingsObjectResponseBodyUnit] = pydantic.Field(default=None)
    """
    The unit of measurement for speeding  Valid values: `milesPerHour`, `kilometersPerHour`, `percentage`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .barcode_value_object_request_body import BarcodeValueObjectRequestBody
from .date_time_value_object_request_body import DateTimeValueObjectRequestBody
from .multiple_choice_value_object_request_body import MultipleChoiceValueObjectRequestBody
from .photo_value_object_request_body import PhotoValueObjectRequestBody
from .scanned_document_value_object_request_body import ScannedDocumentValueObjectRequestBody
from .signature_value_object_request_body import SignatureValueObjectRequestBody


class FieldObjectValueRequestBody(UniversalBaseModel):
    """
    The value of the document field. The shape of value depends on the type.
    """

    barcode_value: typing_extensions.Annotated[
        typing.Optional[typing.List[BarcodeValueObjectRequestBody]],
        FieldMetadata(alias="barcodeValue"),
        pydantic.Field(
            alias="barcodeValue",
            description="The value of a barcode scanning field. Only present for barcode scanning fields.",
        ),
    ] = None
    date_time_value: typing_extensions.Annotated[
        typing.Optional[DateTimeValueObjectRequestBody],
        FieldMetadata(alias="dateTimeValue"),
        pydantic.Field(alias="dateTimeValue"),
    ] = None
    multiple_choice_value: typing_extensions.Annotated[
        typing.Optional[typing.List[MultipleChoiceValueObjectRequestBody]],
        FieldMetadata(alias="multipleChoiceValue"),
        pydantic.Field(
            alias="multipleChoiceValue",
            description="The value of a multiple choice field. Only present for multiple choice fields.",
        ),
    ] = None
    number_value: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="numberValue"),
        pydantic.Field(alias="numberValue", description="The value of a number field. Only present for number fields."),
    ] = None
    photo_value: typing_extensions.Annotated[
        typing.Optional[typing.List[PhotoValueObjectRequestBody]],
        FieldMetadata(alias="photoValue"),
        pydantic.Field(alias="photoValue", description="The value of a photo field. Only present for photo fields."),
    ] = None
    scanned_document_value: typing_extensions.Annotated[
        typing.Optional[typing.List[ScannedDocumentValueObjectRequestBody]],
        FieldMetadata(alias="scannedDocumentValue"),
        pydantic.Field(
            alias="scannedDocumentValue",
            description="The value of a scanned document field. Only present for scanned document fields.",
        ),
    ] = None
    signature_value: typing_extensions.Annotated[
        typing.Optional[SignatureValueObjectRequestBody],
        FieldMetadata(alias="signatureValue"),
        pydantic.Field(alias="signatureValue"),
    ] = None
    string_value: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="stringValue"),
        pydantic.Field(alias="stringValue", description="The value of a string field. Only present for string fields."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .vehicle_external_ids import VehicleExternalIds
from .vehicle_id import VehicleId
from .vehicle_name import VehicleName
from .vehicle_stats_ambient_air_temp_milli_c_with_decoration import VehicleStatsAmbientAirTempMilliCWithDecoration
from .vehicle_stats_aux_input_with_decoration import VehicleStatsAuxInputWithDecoration
from .vehicle_stats_barometric_pressure_pa_with_decoration import VehicleStatsBarometricPressurePaWithDecoration
from .vehicle_stats_battery_voltage_with_decoration import VehicleStatsBatteryVoltageWithDecoration
from .vehicle_stats_def_level_milli_percent_with_decoration import VehicleStatsDefLevelMilliPercentWithDecoration
from .vehicle_stats_ecu_door_status import VehicleStatsEcuDoorStatus
from .vehicle_stats_ecu_speed_mph_with_decoration import VehicleStatsEcuSpeedMphWithDecoration
from .vehicle_stats_engine_coolant_temp_milli_c_with_decoration import VehicleStatsEngineCoolantTempMilliCWithDecoration
from .vehicle_stats_engine_immobilizer_with_decoration import VehicleStatsEngineImmobilizerWithDecoration
from .vehicle_stats_engine_load_percent_with_decoration import VehicleStatsEngineLoadPercentWithDecoration
from .vehicle_stats_engine_oil_pressure_k_pa_with_decoration import VehicleStatsEngineOilPressureKPaWithDecoration
from .vehicle_stats_engine_rpm_with_decoration import VehicleStatsEngineRpmWithDecoration
from .vehicle_stats_engine_state_with_decoration import VehicleStatsEngineStateWithDecoration
from .vehicle_stats_ev_average_battery_temperature_milli_celsius import (
    VehicleStatsEvAverageBatteryTemperatureMilliCelsius,
)
from .vehicle_stats_ev_battery_current_milli_amp import VehicleStatsEvBatteryCurrentMilliAmp
from .vehicle_stats_ev_battery_state_of_health_milli_percent import VehicleStatsEvBatteryStateOfHealthMilliPercent
from .vehicle_stats_ev_battery_voltage_milli_volt import VehicleStatsEvBatteryVoltageMilliVolt
from .vehicle_stats_ev_charging_current_milli_amp import VehicleStatsEvChargingCurrentMilliAmp
from .vehicle_stats_ev_charging_energy_micro_wh import VehicleStatsEvChargingEnergyMicroWh
from .vehicle_stats_ev_charging_status import VehicleStatsEvChargingStatus
from .vehicle_stats_ev_charging_voltage_milli_volt import VehicleStatsEvChargingVoltageMilliVolt
from .vehicle_stats_ev_consumed_energy_micro_wh import VehicleStatsEvConsumedEnergyMicroWh
from .vehicle_stats_ev_distance_driven_meters import VehicleStatsEvDistanceDrivenMeters
from .vehicle_stats_ev_regenerated_energy_micro_wh import VehicleStatsEvRegeneratedEnergyMicroWh
from .vehicle_stats_ev_state_of_charge_milli_percent import VehicleStatsEvStateOfChargeMilliPercent
from .vehicle_stats_fault_codes_with_decoration import VehicleStatsFaultCodesWithDecoration
from .vehicle_stats_fuel_consumed_milliliters_with_decoration import VehicleStatsFuelConsumedMillilitersWithDecoration
from .vehicle_stats_fuel_percent_with_decoration import VehicleStatsFuelPercentWithDecoration
from .vehicle_stats_gps_distance_meters_with_decoration import VehicleStatsGpsDistanceMetersWithDecoration
from .vehicle_stats_gps_odometer_meters_with_decoration import VehicleStatsGpsOdometerMetersWithDecoration
from .vehicle_stats_idling_duration_milliseconds_with_decoration import (
    VehicleStatsIdlingDurationMillisecondsWithDecoration,
)
from .vehicle_stats_intake_manifold_temp_milli_c_with_decoration import (
    VehicleStatsIntakeManifoldTempMilliCWithDecoration,
)
from .vehicle_stats_list_gps import VehicleStatsListGps
from .vehicle_stats_list_synthetic_engine_seconds import VehicleStatsListSyntheticEngineSeconds
from .vehicle_stats_nfc_card_scan_with_decoration import VehicleStatsNfcCardScanWithDecoration
from .vehicle_stats_obd_engine_seconds_with_decoration import VehicleStatsObdEngineSecondsWithDecoration
from .vehicle_stats_obd_odometer_meters_with_decoration import VehicleStatsObdOdometerMetersWithDecoration
from .vehicle_stats_seatbelt_driver import VehicleStatsSeatbeltDriver
from .vehicle_stats_spreader_active import VehicleStatsSpreaderActive
from .vehicle_stats_spreader_air_temp import VehicleStatsSpreaderAirTemp
from .vehicle_stats_spreader_blast_state import VehicleStatsSpreaderBlastState
from .vehicle_stats_spreader_granular_name import VehicleStatsSpreaderGranularName
from .vehicle_stats_spreader_granular_rate import VehicleStatsSpreaderGranularRate
from .vehicle_stats_spreader_liquid_name import VehicleStatsSpreaderLiquidName
from .vehicle_stats_spreader_liquid_rate import VehicleStatsSpreaderLiquidRate
from .vehicle_stats_spreader_on_state import VehicleStatsSpreaderOnState
from .vehicle_stats_spreader_plow_status import VehicleStatsSpreaderPlowStatus
from .vehicle_stats_spreader_prewet_name import VehicleStatsSpreaderPrewetName
from .vehicle_stats_spreader_prewet_rate import VehicleStatsSpreaderPrewetRate
from .vehicle_stats_spreader_road_temp import VehicleStatsSpreaderRoadTemp


class VehicleStatsListResponseData(UniversalBaseModel):
    """
    A vehicle and its list of stat events.
    """

    ambient_air_temperature_milli_c: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsAmbientAirTempMilliCWithDecoration]],
        FieldMetadata(alias="ambientAirTemperatureMilliC"),
        pydantic.Field(
            alias="ambientAirTemperatureMilliC",
            description="A list of ambient air temperature readings in millidegree Celsius for the given vehicle.",
        ),
    ] = None
    aux_input_1: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsAuxInputWithDecoration]],
        FieldMetadata(alias="auxInput1"),
        pydantic.Field(alias="auxInput1", description="A list of auxiliary equipment states."),
    ] = None
    aux_input_10: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsAuxInputWithDecoration]],
        FieldMetadata(alias="auxInput10"),
        pydantic.Field(alias="auxInput10", description="A list of auxiliary equipment states."),
    ] = None
    aux_input_11: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsAuxInputWithDecoration]],
        FieldMetadata(alias="auxInput11"),
        pydantic.Field(alias="auxInput11", description="A list of auxiliary equipment states."),
    ] = None
    aux_input_12: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsAuxInputWithDecoration]],
        FieldMetadata(alias="auxInput12"),
        pydantic.Field(alias="auxInput12", description="A list of auxiliary equipment states."),
    ] = None
    aux_input_13: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsAuxInputWithDecoration]],
        FieldMetadata(alias="auxInput13"),
        pydantic.Field(alias="auxInput13", description="A list of auxiliary equipment states."),
    ] = None
    aux_input_2: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsAuxInputWithDecoration]],
        FieldMetadata(alias="auxInput2"),
        pydantic.Field(alias="auxInput2", description="A list of auxiliary equipment states."),
    ] = None
    aux_input_3: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsAuxInputWithDecoration]],
        FieldMetadata(alias="auxInput3"),
        pydantic.Field(alias="auxInput3", description="A list of auxiliary equipment states."),
    ] = None
    aux_input_4: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsAuxInputWithDecoration]],
        FieldMetadata(alias="auxInput4"),
        pydantic.Field(alias="auxInput4", description="A list of auxiliary equipment states."),
    ] = None
    aux_input_5: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsAuxInputWithDecoration]],
        FieldMetadata(alias="auxInput5"),
        pydantic.Field(alias="auxInput5", description="A list of auxiliary equipment states."),
    ] = None
    aux_input_6: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsAuxInputWithDecoration]],
        FieldMetadata(alias="auxInput6"),
        pydantic.Field(alias="auxInput6", description="A list of auxiliary equipment states."),
    ] = None
    aux_input_7: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsAuxInputWithDecoration]],
        FieldMetadata(alias="auxInput7"),
        pydantic.Field(alias="auxInput7", description="A list of auxiliary equipment states."),
    ] = None
    aux_input_8: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsAuxInputWithDecoration]],
        FieldMetadata(alias="auxInput8"),
        pydantic.Field(alias="auxInput8", description="A list of auxiliary equipment states."),
    ] = None
    aux_input_9: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsAuxInputWithDecoration]],
        FieldMetadata(alias="auxInput9"),
        pydantic.Field(alias="auxInput9", description="A list of auxiliary equipment states."),
    ] = None
    barometric_pressure_pa: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsBarometricPressurePaWithDecoration]],
        FieldMetadata(alias="barometricPressurePa"),
        pydantic.Field(
            alias="barometricPressurePa",
            description="A list of barometric pressure readings in pascals for the given vehicle.",
        ),
    ] = None
    battery_milli_volts: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsBatteryVoltageWithDecoration]],
        FieldMetadata(alias="batteryMilliVolts"),
        pydantic.Field(
            alias="batteryMilliVolts", description="A list of battery levels in milliVolts for the given vehicle."
        ),
    ] = None
    def_level_milli_percent: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsDefLevelMilliPercentWithDecoration]],
        FieldMetadata(alias="defLevelMilliPercent"),
        pydantic.Field(
            alias="defLevelMilliPercent",
            description="A list of DEF level milli percentage readings for the given vehicle.",
        ),
    ] = None
    ecu_door_status: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsEcuDoorStatus]],
        FieldMetadata(alias="ecuDoorStatus"),
        pydantic.Field(
            alias="ecuDoorStatus",
            description="Door status as read from the vehicle (either from ECU or AUX as a fallback).",
        ),
    ] = None
    ecu_speed_mph: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsEcuSpeedMphWithDecoration]],
        FieldMetadata(alias="ecuSpeedMph"),
        pydantic.Field(
            alias="ecuSpeedMph",
            description="A list of the speeds of the vehicle in miles per hour, as reported by the ECU.",
        ),
    ] = None
    engine_coolant_temperature_milli_c: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsEngineCoolantTempMilliCWithDecoration]],
        FieldMetadata(alias="engineCoolantTemperatureMilliC"),
        pydantic.Field(
            alias="engineCoolantTemperatureMilliC",
            description="A list of engine coolant temperature readings in millidegree Celsius for the given vehicle.",
        ),
    ] = None
    engine_immobilizer: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsEngineImmobilizerWithDecoration]],
        FieldMetadata(alias="engineImmobilizer"),
        pydantic.Field(
            alias="engineImmobilizer", description="A list of engine immobilizer data for the given vehicle."
        ),
    ] = None
    engine_load_percent: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsEngineLoadPercentWithDecoration]],
        FieldMetadata(alias="engineLoadPercent"),
        pydantic.Field(
            alias="engineLoadPercent", description="A list of engine load percentage readings for the given vehicle."
        ),
    ] = None
    engine_oil_pressure_k_pa: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsEngineOilPressureKPaWithDecoration]],
        FieldMetadata(alias="engineOilPressureKPa"),
        pydantic.Field(
            alias="engineOilPressureKPa",
            description="A list of engine oil pressure readings in kilopascals for the given vehicle.",
        ),
    ] = None
    engine_rpm: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsEngineRpmWithDecoration]],
        FieldMetadata(alias="engineRpm"),
        pydantic.Field(alias="engineRpm", description="A list engine RPM values for the given vehicle."),
    ] = None
    engine_states: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsEngineStateWithDecoration]],
        FieldMetadata(alias="engineStates"),
        pydantic.Field(alias="engineStates", description="A list of engine state events for the given vehicle."),
    ] = None
    ev_average_battery_temperature_milli_celsius: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsEvAverageBatteryTemperatureMilliCelsius]],
        FieldMetadata(alias="evAverageBatteryTemperatureMilliCelsius"),
        pydantic.Field(
            alias="evAverageBatteryTemperatureMilliCelsius",
            description="Battery temperature for electric and hybrid vehicles in milli celsius. Not all EV and HEVs may report this field.",
        ),
    ] = None
    ev_battery_current_milli_amp: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsEvBatteryCurrentMilliAmp]],
        FieldMetadata(alias="evBatteryCurrentMilliAmp"),
        pydantic.Field(
            alias="evBatteryCurrentMilliAmp",
            description="Battery current for electric and hybrid vehicles in milli amps. Not all EV and HEVs may report this field.",
        ),
    ] = None
    ev_battery_state_of_health_milli_percent: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsEvBatteryStateOfHealthMilliPercent]],
        FieldMetadata(alias="evBatteryStateOfHealthMilliPercent"),
        pydantic.Field(
            alias="evBatteryStateOfHealthMilliPercent",
            description="Milli percent battery state of health for electric and hybrid vehicles. Not all EV and HEVs may report this field.",
        ),
    ] = None
    ev_battery_voltage_milli_volt: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsEvBatteryVoltageMilliVolt]],
        FieldMetadata(alias="evBatteryVoltageMilliVolt"),
        pydantic.Field(
            alias="evBatteryVoltageMilliVolt",
            description="Battery voltage for electric and hybrid vehicles in milli volts. Not all EV and HEVs may report this field.",
        ),
    ] = None
    ev_charging_current_milli_amp: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsEvChargingCurrentMilliAmp]],
        FieldMetadata(alias="evChargingCurrentMilliAmp"),
        pydantic.Field(
            alias="evChargingCurrentMilliAmp",
            description="Charging current for electric and hybrid vehicles in milli amps. Not all EV and HEVs may report this field.",
        ),
    ] = None
    ev_charging_energy_micro_wh: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsEvChargingEnergyMicroWh]],
        FieldMetadata(alias="evChargingEnergyMicroWh"),
        pydantic.Field(
            alias="evChargingEnergyMicroWh",
            description="Charging energy for electric and hybrid vehicles in microwatt hours. Not all EV and HEVs may report this field.",
        ),
    ] = None
    ev_charging_status: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsEvChargingStatus]],
        FieldMetadata(alias="evChargingStatus"),
        pydantic.Field(
            alias="evChargingStatus",
            description="Charging status for electric and hybrid vehicles. Not all EV and HEVs may report this field.",
        ),
    ] = None
    ev_charging_voltage_milli_volt: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsEvChargingVoltageMilliVolt]],
        FieldMetadata(alias="evChargingVoltageMilliVolt"),
        pydantic.Field(
            alias="evChargingVoltageMilliVolt",
            description="Charging voltage for electric and hybrid vehicles in milli  volts. Not all EV and HEVs may report this field.",
        ),
    ] = None
    ev_consumed_energy_micro_wh: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsEvConsumedEnergyMicroWh]],
        FieldMetadata(alias="evConsumedEnergyMicroWh"),
        pydantic.Field(
            alias="evConsumedEnergyMicroWh",
            description="Consumed energy (including regenerated) for electric and hybrid vehicles in microwatt hours. Not all EV and HEVs may report this field.",
        ),
    ] = None
    ev_distance_driven_meters: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsEvDistanceDrivenMeters]],
        FieldMetadata(alias="evDistanceDrivenMeters"),
        pydantic.Field(
            alias="evDistanceDrivenMeters",
            description="Electric distance driven for electric and hybrid vehicles in meters. Not all EV and HEVs may report this field.",
        ),
    ] = None
    ev_regenerated_energy_micro_wh: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsEvRegeneratedEnergyMicroWh]],
        FieldMetadata(alias="evRegeneratedEnergyMicroWh"),
        pydantic.Field(
            alias="evRegeneratedEnergyMicroWh",
            description="Regenerated energy for electric and hybrid vehicles in microwatt hours. Not all EV and HEVs may report this field.",
        ),
    ] = None
    ev_state_of_charge_milli_percent: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsEvStateOfChargeMilliPercent]],
        FieldMetadata(alias="evStateOfChargeMilliPercent"),
        pydantic.Field(
            alias="evStateOfChargeMilliPercent",
            description="State of Charge for electric and hybrid vehicles. Not all EV and HEVs may report this field.",
        ),
    ] = None
    external_ids: typing_extensions.Annotated[
        typing.Optional[VehicleExternalIds], FieldMetadata(alias="externalIds"), pydantic.Field(alias="externalIds")
    ] = None
    fault_codes: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsFaultCodesWithDecoration]],
        FieldMetadata(alias="faultCodes"),
        pydantic.Field(alias="faultCodes", description="A list of engine fault codes."),
    ] = None
    fuel_consumed_milliliters: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsFuelConsumedMillilitersWithDecoration]],
        FieldMetadata(alias="fuelConsumedMilliliters"),
        pydantic.Field(
            alias="fuelConsumedMilliliters",
            description='A list of cumulative fuel consumption readings in milliliters for the given vehicle. Cumulative values always increase. This includes all fuel consumption reported by vehicles without filtering of invalid data points. For filtered fuel consumption that matches the Fuel & Energy Report, please use <a href="https://developers.samsara.com/reference/getfuelenergyvehiclereports" target="_blank">the Fuel and Energy API</a>.',
        ),
    ] = None
    fuel_percents: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsFuelPercentWithDecoration]],
        FieldMetadata(alias="fuelPercents"),
        pydantic.Field(alias="fuelPercents", description="A list of fuel percentage readings for the given vehicle."),
    ] = None
    gps: typing.Optional[typing.List[VehicleStatsListGps]] = pydantic.Field(default=None)
    """
    A list of GPS location events for the given vehicles.
    """

    gps_distance_meters: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsGpsDistanceMetersWithDecoration]],
        FieldMetadata(alias="gpsDistanceMeters"),
        pydantic.Field(alias="gpsDistanceMeters", description="A list of GPS distance events for the given vehicle."),
    ] = None
    gps_odometer_meters: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsGpsOdometerMetersWithDecoration]],
        FieldMetadata(alias="gpsOdometerMeters"),
        pydantic.Field(alias="gpsOdometerMeters", description="A list of GPS odometer events for the given vehicle."),
    ] = None
    id: typing.Optional[VehicleId] = None
    idling_duration_milliseconds: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsIdlingDurationMillisecondsWithDecoration]],
        FieldMetadata(alias="idlingDurationMilliseconds"),
        pydantic.Field(
            alias="idlingDurationMilliseconds",
            description='A list of cumulative idling duration readings in milliseconds for the given vehicle. Cumulative values always increase. For filtering of idling duration please use <a href="https://developers.samsara.com/reference/getvehicleidlingreports" target="_blank">the Idling Events API</a>.',
        ),
    ] = None
    intake_manifold_temperature_milli_c: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsIntakeManifoldTempMilliCWithDecoration]],
        FieldMetadata(alias="intakeManifoldTemperatureMilliC"),
        pydantic.Field(
            alias="intakeManifoldTemperatureMilliC",
            description="A list of intake manifold temperature readings in millidegree Celsius for the given vehicle.",
        ),
    ] = None
    name: typing.Optional[VehicleName] = None
    nfc_card_scans: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsNfcCardScanWithDecoration]],
        FieldMetadata(alias="nfcCardScans"),
        pydantic.Field(
            alias="nfcCardScans", description="A list of NFC cards that were scanned for the given vehicles."
        ),
    ] = None
    obd_engine_seconds: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsObdEngineSecondsWithDecoration]],
        FieldMetadata(alias="obdEngineSeconds"),
        pydantic.Field(
            alias="obdEngineSeconds", description="A list of OBD engine seconds readings for the given vehicle."
        ),
    ] = None
    obd_odometer_meters: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsObdOdometerMetersWithDecoration]],
        FieldMetadata(alias="obdOdometerMeters"),
        pydantic.Field(alias="obdOdometerMeters", description="A list of OBD odometer readings for the given vehicle."),
    ] = None
    seatbelt_driver: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsSeatbeltDriver]],
        FieldMetadata(alias="seatbeltDriver"),
        pydantic.Field(
            alias="seatbeltDriver",
            description="Seatbelt Driver Status as read from the vehicle. `Buckled` or `Unbuckled`.",
        ),
    ] = None
    spreader_active: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsSpreaderActive]],
        FieldMetadata(alias="spreaderActive"),
        pydantic.Field(alias="spreaderActive", description="Whether vehicle is actively spreading any material."),
    ] = None
    spreader_air_temp: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsSpreaderAirTemp]],
        FieldMetadata(alias="spreaderAirTemp"),
        pydantic.Field(
            alias="spreaderAirTemp",
            description="Air (ambient) temperature in milli celsius reading from material spreader.",
        ),
    ] = None
    spreader_blast_state: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsSpreaderBlastState]],
        FieldMetadata(alias="spreaderBlastState"),
        pydantic.Field(
            alias="spreaderBlastState", description="Whether vehicle is actively spreading material in ‘blast mode’."
        ),
    ] = None
    spreader_granular_name: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsSpreaderGranularName]],
        FieldMetadata(alias="spreaderGranularName"),
        pydantic.Field(
            alias="spreaderGranularName",
            description="Name of most recent type of granular material spread, read from the material spreader.",
        ),
    ] = None
    spreader_granular_rate: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsSpreaderGranularRate]],
        FieldMetadata(alias="spreaderGranularRate"),
        pydantic.Field(
            alias="spreaderGranularRate",
            description="Granular spread rate reading in milliliters per meter, read from the material spreader. Unfiltered live stats are supplied as-read from the Material Spreader unit. Readings do not consider total spread rate(s) over time or distance.",
        ),
    ] = None
    spreader_liquid_name: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsSpreaderLiquidName]],
        FieldMetadata(alias="spreaderLiquidName"),
        pydantic.Field(
            alias="spreaderLiquidName",
            description="Name of most recent type of liquid material spread, read from the material spreader.",
        ),
    ] = None
    spreader_liquid_rate: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsSpreaderLiquidRate]],
        FieldMetadata(alias="spreaderLiquidRate"),
        pydantic.Field(
            alias="spreaderLiquidRate",
            description="Liquid spread rate reading in milliliters per meter, read from the material spreader. Unfiltered live stats are supplied as-read from the Material Spreader unit. Readings do not consider total spread rate(s) over time or distance.",
        ),
    ] = None
    spreader_on_state: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsSpreaderOnState]],
        FieldMetadata(alias="spreaderOnState"),
        pydantic.Field(alias="spreaderOnState", description="Whether vehicle spreader is enabled."),
    ] = None
    spreader_plow_status: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsSpreaderPlowStatus]],
        FieldMetadata(alias="spreaderPlowStatus"),
        pydantic.Field(
            alias="spreaderPlowStatus",
            description="Snow plow status (`Up` or `Down`), as read from the material spreader. Note: this is separate from plow status defined via auxInput.",
        ),
    ] = None
    spreader_prewet_name: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsSpreaderPrewetName]],
        FieldMetadata(alias="spreaderPrewetName"),
        pydantic.Field(
            alias="spreaderPrewetName",
            description="Name of most recent type of prewet material spread, read from the material spreader.",
        ),
    ] = None
    spreader_prewet_rate: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsSpreaderPrewetRate]],
        FieldMetadata(alias="spreaderPrewetRate"),
        pydantic.Field(
            alias="spreaderPrewetRate",
            description="Prewet spread rate reading in milliliters per meter, read from the material spreader. Unfiltered live stats are supplied as-read from the Material Spreader unit. Readings do not consider total spread rate(s) over time or distance.",
        ),
    ] = None
    spreader_road_temp: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsSpreaderRoadTemp]],
        FieldMetadata(alias="spreaderRoadTemp"),
        pydantic.Field(
            alias="spreaderRoadTemp", description="Road temperature reading in milli celsius from material spreader."
        ),
    ] = None
    synthetic_engine_seconds: typing_extensions.Annotated[
        typing.Optional[typing.List[VehicleStatsListSyntheticEngineSeconds]],
        FieldMetadata(alias="syntheticEngineSeconds"),
        pydantic.Field(alias="syntheticEngineSeconds", description="A list of synthetic engine seconds values."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_gateway_error import BadGatewayError
from ..errors.gateway_timeout_error import GatewayTimeoutError
from ..errors.internal_server_error import InternalServerError
from ..errors.method_not_allowed_error import MethodNotAllowedError
from ..errors.not_found_error import NotFoundError
from ..errors.not_implemented_error import NotImplementedError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.driver_vehicle_assignments_v_2_create_driver_vehicle_assignment_response_body import (
    DriverVehicleAssignmentsV2CreateDriverVehicleAssignmentResponseBody,
)
from ..types.driver_vehicle_assignments_v_2_get_driver_vehicle_assignments_response_body import (
    DriverVehicleAssignmentsV2GetDriverVehicleAssignmentsResponseBody,
)
from ..types.driver_vehicle_assignments_v_2_update_driver_vehicle_assignment_response_body import (
    DriverVehicleAssignmentsV2UpdateDriverVehicleAssignmentResponseBody,
)
from ..types.patch_driver_vehicle_assignments_v_2_request_body_metadata_request_body import (
    PatchDriverVehicleAssignmentsV2RequestBodyMetadataRequestBody,
)
from ..types.post_driver_vehicle_assignments_v_2_request_body_metadata_request_body import (
    PostDriverVehicleAssignmentsV2RequestBodyMetadataRequestBody,
)
from .types.get_driver_vehicle_assignments_request_assignment_type import (
    GetDriverVehicleAssignmentsRequestAssignmentType,
)
from .types.get_driver_vehicle_assignments_request_filter_by import GetDriverVehicleAssignmentsRequestFilterBy

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawDriverVehicleAssignmentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_driver_vehicle_assignments(
        self,
        *,
        filter_by: GetDriverVehicleAssignmentsRequestFilterBy,
        start_time: typing.Optional[str] = None,
        end_time: typing.Optional[str] = None,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        vehicle_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        driver_tag_ids: typing.Optional[str] = None,
        vehicle_tag_ids: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        assignment_type: typing.Optional[GetDriverVehicleAssignmentsRequestAssignmentType] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[DriverVehicleAssignmentsV2GetDriverVehicleAssignmentsResponseBody]:
        """
        Get all driver-vehicle assignments for the requested drivers or vehicles in the requested time range. To fetch driver-vehicle assignments out of the vehicle trips' time ranges, assignmentType needs to be specified. Note: this endpoint replaces past endpoints to fetch assignments by driver or by vehicle. Visit [this migration guide](https://developers.samsara.com/docs/migrating-from-driver-vehicle-assignment-or-vehicle-driver-assignment-endpoints) for more information.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        filter_by : GetDriverVehicleAssignmentsRequestFilterBy
            Option to filter by drivers or vehicles.  Valid values: `drivers`, `vehicles`

        start_time : typing.Optional[str]
             A start time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : typing.Optional[str]
             An end time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
             A filter on the data based on this comma-separated list of driver IDs and externalIds. Example: `driverIds=1234,5678,payroll:4841`

        vehicle_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            ID of the vehicle. This can either be the Samsara-specified ID, or an external ID. External IDs are customer specified key-value pairs created in the POST or PATCH requests of this resource. To specify an external ID as part of a path parameter, use the following format: "key:value". For example, "maintenanceId:250020".

        driver_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of driver tag IDs. Example: `tagIds=1234,5678`

        vehicle_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of vehicle tag IDs. Example: `tagIds=1234,5678`

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        assignment_type : typing.Optional[GetDriverVehicleAssignmentsRequestAssignmentType]
            Specifies which assignment type to filter by.  Valid values: `HOS`, `idCard`, `static`, `faceId`, `tachograph`, `safetyManual`, `RFID`, `trailer`, `external`, `qrCode`, `driverApp`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DriverVehicleAssignmentsV2GetDriverVehicleAssignmentsResponseBody]
            OK response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "fleet/driver-vehicle-assignments",
            method="GET",
            params={
                "filterBy": filter_by,
                "startTime": start_time,
                "endTime": end_time,
                "driverIds": driver_ids,
                "vehicleIds": vehicle_ids,
                "driverTagIds": driver_tag_ids,
                "vehicleTagIds": vehicle_tag_ids,
                "after": after,
                "assignmentType": assignment_type,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DriverVehicleAssignmentsV2GetDriverVehicleAssignmentsResponseBody,
                    parse_obj_as(
                        type_=DriverVehicleAssignmentsV2GetDriverVehicleAssignmentsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_driver_vehicle_assignment(
        self,
        *,
        driver_id: str,
        vehicle_id: str,
        assigned_at_time: typing.Optional[str] = OMIT,
        end_time: typing.Optional[str] = OMIT,
        is_passenger: typing.Optional[bool] = OMIT,
        metadata: typing.Optional[PostDriverVehicleAssignmentsV2RequestBodyMetadataRequestBody] = OMIT,
        start_time: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[DriverVehicleAssignmentsV2CreateDriverVehicleAssignmentResponseBody]:
        """
        Assign vehicle drive-time to a driver via API. For a step-by-step instruction on how to leverage this endpoint, see [this guide](https://developers.samsara.com/docs/creating-driver-vehicle-assignments)

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_id : str
            ID of the driver. This can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the driver.

        vehicle_id : str
            ID of the vehicle. This can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the vehicle.

        assigned_at_time : typing.Optional[str]
            The time at which the assignment was made in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : typing.Optional[str]
            The end time in RFC 3339 format. Defaults to max-time (meaning it's an ongoing assignment) if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        is_passenger : typing.Optional[bool]
            Is this driver a passenger? Defaults to false if not provided

        metadata : typing.Optional[PostDriverVehicleAssignmentsV2RequestBodyMetadataRequestBody]

        start_time : typing.Optional[str]
            The start time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DriverVehicleAssignmentsV2CreateDriverVehicleAssignmentResponseBody]
            Created response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "fleet/driver-vehicle-assignments",
            method="POST",
            json={
                "assignedAtTime": assigned_at_time,
                "driverId": driver_id,
                "endTime": end_time,
                "isPassenger": is_passenger,
                "metadata": convert_and_respect_annotation_metadata(
                    object_=metadata,
                    annotation=PostDriverVehicleAssignmentsV2RequestBodyMetadataRequestBody,
                    direction="write",
                ),
                "startTime": start_time,
                "vehicleId": vehicle_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DriverVehicleAssignmentsV2CreateDriverVehicleAssignmentResponseBody,
                    parse_obj_as(
                        type_=DriverVehicleAssignmentsV2CreateDriverVehicleAssignmentResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_driver_vehicle_assignments(
        self,
        *,
        vehicle_id: str,
        assigned_at_time: typing.Optional[str] = OMIT,
        end_time: typing.Optional[str] = OMIT,
        is_passenger: typing.Optional[bool] = OMIT,
        start_time: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[None]:
        """
        Delete driver assignments that were created using the `POST fleet/driver-vehicle-assignments` endpoint for the requested vehicle in the requested time range.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        vehicle_id : str
            ID of the vehicle. This can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the vehicle.

        assigned_at_time : typing.Optional[str]
             Assigned at time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : typing.Optional[str]
             An end time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        is_passenger : typing.Optional[bool]
            Indicates if assigned driver is passenger

        start_time : typing.Optional[str]
             A start time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            "fleet/driver-vehicle-assignments",
            method="DELETE",
            json={
                "assignedAtTime": assigned_at_time,
                "endTime": end_time,
                "isPassenger": is_passenger,
                "startTime": start_time,
                "vehicleId": vehicle_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_driver_vehicle_assignment(
        self,
        *,
        driver_id: str,
        start_time: str,
        vehicle_id: str,
        assigned_at_time: typing.Optional[str] = OMIT,
        end_time: typing.Optional[str] = OMIT,
        is_passenger: typing.Optional[bool] = OMIT,
        metadata: typing.Optional[PatchDriverVehicleAssignmentsV2RequestBodyMetadataRequestBody] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[DriverVehicleAssignmentsV2UpdateDriverVehicleAssignmentResponseBody]:
        """
        Update driver assignments that were created using the `POST fleet/driver-vehicle-assignments`. Vehicle Id, Driver Id, and Start Time must match an existing assignment.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_id : str
            ID of the driver. This can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the driver.

        start_time : str
            The start time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        vehicle_id : str
            ID of the vehicle. This can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the vehicle.

        assigned_at_time : typing.Optional[str]
            The time at which the assignment was made in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : typing.Optional[str]
            The end time in RFC 3339 format. To make this an ongoing assignment (ie. an assignment with no end time), provide an endTime value of 'null'. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        is_passenger : typing.Optional[bool]
            Is this driver a passenger?

        metadata : typing.Optional[PatchDriverVehicleAssignmentsV2RequestBodyMetadataRequestBody]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DriverVehicleAssignmentsV2UpdateDriverVehicleAssignmentResponseBody]
            Accepted response.
        """
        _response = self._client_wrapper.httpx_client.request(
            "fleet/driver-vehicle-assignments",
            method="PATCH",
            json={
                "assignedAtTime": assigned_at_time,
                "driverId": driver_id,
                "endTime": end_time,
                "isPassenger": is_passenger,
                "metadata": convert_and_respect_annotation_metadata(
                    object_=metadata,
                    annotation=PatchDriverVehicleAssignmentsV2RequestBodyMetadataRequestBody,
                    direction="write",
                ),
                "startTime": start_time,
                "vehicleId": vehicle_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DriverVehicleAssignmentsV2UpdateDriverVehicleAssignmentResponseBody,
                    parse_obj_as(
                        type_=DriverVehicleAssignmentsV2UpdateDriverVehicleAssignmentResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawDriverVehicleAssignmentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_driver_vehicle_assignments(
        self,
        *,
        filter_by: GetDriverVehicleAssignmentsRequestFilterBy,
        start_time: typing.Optional[str] = None,
        end_time: typing.Optional[str] = None,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        vehicle_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        driver_tag_ids: typing.Optional[str] = None,
        vehicle_tag_ids: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        assignment_type: typing.Optional[GetDriverVehicleAssignmentsRequestAssignmentType] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[DriverVehicleAssignmentsV2GetDriverVehicleAssignmentsResponseBody]:
        """
        Get all driver-vehicle assignments for the requested drivers or vehicles in the requested time range. To fetch driver-vehicle assignments out of the vehicle trips' time ranges, assignmentType needs to be specified. Note: this endpoint replaces past endpoints to fetch assignments by driver or by vehicle. Visit [this migration guide](https://developers.samsara.com/docs/migrating-from-driver-vehicle-assignment-or-vehicle-driver-assignment-endpoints) for more information.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        filter_by : GetDriverVehicleAssignmentsRequestFilterBy
            Option to filter by drivers or vehicles.  Valid values: `drivers`, `vehicles`

        start_time : typing.Optional[str]
             A start time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : typing.Optional[str]
             An end time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
             A filter on the data based on this comma-separated list of driver IDs and externalIds. Example: `driverIds=1234,5678,payroll:4841`

        vehicle_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            ID of the vehicle. This can either be the Samsara-specified ID, or an external ID. External IDs are customer specified key-value pairs created in the POST or PATCH requests of this resource. To specify an external ID as part of a path parameter, use the following format: "key:value". For example, "maintenanceId:250020".

        driver_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of driver tag IDs. Example: `tagIds=1234,5678`

        vehicle_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of vehicle tag IDs. Example: `tagIds=1234,5678`

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        assignment_type : typing.Optional[GetDriverVehicleAssignmentsRequestAssignmentType]
            Specifies which assignment type to filter by.  Valid values: `HOS`, `idCard`, `static`, `faceId`, `tachograph`, `safetyManual`, `RFID`, `trailer`, `external`, `qrCode`, `driverApp`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DriverVehicleAssignmentsV2GetDriverVehicleAssignmentsResponseBody]
            OK response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fleet/driver-vehicle-assignments",
            method="GET",
            params={
                "filterBy": filter_by,
                "startTime": start_time,
                "endTime": end_time,
                "driverIds": driver_ids,
                "vehicleIds": vehicle_ids,
                "driverTagIds": driver_tag_ids,
                "vehicleTagIds": vehicle_tag_ids,
                "after": after,
                "assignmentType": assignment_type,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DriverVehicleAssignmentsV2GetDriverVehicleAssignmentsResponseBody,
                    parse_obj_as(
                        type_=DriverVehicleAssignmentsV2GetDriverVehicleAssignmentsResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_driver_vehicle_assignment(
        self,
        *,
        driver_id: str,
        vehicle_id: str,
        assigned_at_time: typing.Optional[str] = OMIT,
        end_time: typing.Optional[str] = OMIT,
        is_passenger: typing.Optional[bool] = OMIT,
        metadata: typing.Optional[PostDriverVehicleAssignmentsV2RequestBodyMetadataRequestBody] = OMIT,
        start_time: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[DriverVehicleAssignmentsV2CreateDriverVehicleAssignmentResponseBody]:
        """
        Assign vehicle drive-time to a driver via API. For a step-by-step instruction on how to leverage this endpoint, see [this guide](https://developers.samsara.com/docs/creating-driver-vehicle-assignments)

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_id : str
            ID of the driver. This can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the driver.

        vehicle_id : str
            ID of the vehicle. This can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the vehicle.

        assigned_at_time : typing.Optional[str]
            The time at which the assignment was made in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : typing.Optional[str]
            The end time in RFC 3339 format. Defaults to max-time (meaning it's an ongoing assignment) if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        is_passenger : typing.Optional[bool]
            Is this driver a passenger? Defaults to false if not provided

        metadata : typing.Optional[PostDriverVehicleAssignmentsV2RequestBodyMetadataRequestBody]

        start_time : typing.Optional[str]
            The start time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DriverVehicleAssignmentsV2CreateDriverVehicleAssignmentResponseBody]
            Created response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fleet/driver-vehicle-assignments",
            method="POST",
            json={
                "assignedAtTime": assigned_at_time,
                "driverId": driver_id,
                "endTime": end_time,
                "isPassenger": is_passenger,
                "metadata": convert_and_respect_annotation_metadata(
                    object_=metadata,
                    annotation=PostDriverVehicleAssignmentsV2RequestBodyMetadataRequestBody,
                    direction="write",
                ),
                "startTime": start_time,
                "vehicleId": vehicle_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DriverVehicleAssignmentsV2CreateDriverVehicleAssignmentResponseBody,
                    parse_obj_as(
                        type_=DriverVehicleAssignmentsV2CreateDriverVehicleAssignmentResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_driver_vehicle_assignments(
        self,
        *,
        vehicle_id: str,
        assigned_at_time: typing.Optional[str] = OMIT,
        end_time: typing.Optional[str] = OMIT,
        is_passenger: typing.Optional[bool] = OMIT,
        start_time: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[None]:
        """
        Delete driver assignments that were created using the `POST fleet/driver-vehicle-assignments` endpoint for the requested vehicle in the requested time range.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        vehicle_id : str
            ID of the vehicle. This can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the vehicle.

        assigned_at_time : typing.Optional[str]
             Assigned at time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : typing.Optional[str]
             An end time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        is_passenger : typing.Optional[bool]
            Indicates if assigned driver is passenger

        start_time : typing.Optional[str]
             A start time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fleet/driver-vehicle-assignments",
            method="DELETE",
            json={
                "assignedAtTime": assigned_at_time,
                "endTime": end_time,
                "isPassenger": is_passenger,
                "startTime": start_time,
                "vehicleId": vehicle_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_driver_vehicle_assignment(
        self,
        *,
        driver_id: str,
        start_time: str,
        vehicle_id: str,
        assigned_at_time: typing.Optional[str] = OMIT,
        end_time: typing.Optional[str] = OMIT,
        is_passenger: typing.Optional[bool] = OMIT,
        metadata: typing.Optional[PatchDriverVehicleAssignmentsV2RequestBodyMetadataRequestBody] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[DriverVehicleAssignmentsV2UpdateDriverVehicleAssignmentResponseBody]:
        """
        Update driver assignments that were created using the `POST fleet/driver-vehicle-assignments`. Vehicle Id, Driver Id, and Start Time must match an existing assignment.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_id : str
            ID of the driver. This can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the driver.

        start_time : str
            The start time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        vehicle_id : str
            ID of the vehicle. This can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the vehicle.

        assigned_at_time : typing.Optional[str]
            The time at which the assignment was made in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : typing.Optional[str]
            The end time in RFC 3339 format. To make this an ongoing assignment (ie. an assignment with no end time), provide an endTime value of 'null'. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        is_passenger : typing.Optional[bool]
            Is this driver a passenger?

        metadata : typing.Optional[PatchDriverVehicleAssignmentsV2RequestBodyMetadataRequestBody]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DriverVehicleAssignmentsV2UpdateDriverVehicleAssignmentResponseBody]
            Accepted response.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fleet/driver-vehicle-assignments",
            method="PATCH",
            json={
                "assignedAtTime": assigned_at_time,
                "driverId": driver_id,
                "endTime": end_time,
                "isPassenger": is_passenger,
                "metadata": convert_and_respect_annotation_metadata(
                    object_=metadata,
                    annotation=PatchDriverVehicleAssignmentsV2RequestBodyMetadataRequestBody,
                    direction="write",
                ),
                "startTime": start_time,
                "vehicleId": vehicle_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DriverVehicleAssignmentsV2UpdateDriverVehicleAssignmentResponseBody,
                    parse_obj_as(
                        type_=DriverVehicleAssignmentsV2UpdateDriverVehicleAssignmentResponseBody,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

HealthReasonResponseResponseBodyHealthReasonCode = typing.Union[
    typing.Literal[
        "assetUnplugged",
        "cameraMisaligned",
        "dataPending",
        "healthy",
        "inwardCameraObstruction",
        "irregularRecording",
        "lowDeviceBattery",
        "missingEldDiagnostics",
        "missingVin",
        "needsReplacement",
        "newlyInstalledDevice",
        "noGpsSignal",
        "notDetected",
        "outwardCameraObstruction",
        "recordingTimeRequired",
        "unknown",
        "vgUnplugged",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .forms_asset_object_response_body import FormsAssetObjectResponseBody
from .forms_media_record_object_response_body import FormsMediaRecordObjectResponseBody
from .forms_polymorphic_user_object_response_body import FormsPolymorphicUserObjectResponseBody
from .issue_response_object_response_body_priority import IssueResponseObjectResponseBodyPriority
from .issue_response_object_response_body_status import IssueResponseObjectResponseBodyStatus
from .issue_source_object_response_body import IssueSourceObjectResponseBody


class IssueResponseObjectResponseBody(UniversalBaseModel):
    """
    Issue response object.
    """

    asset: typing.Optional[FormsAssetObjectResponseBody] = None
    assigned_to: typing_extensions.Annotated[
        typing.Optional[FormsPolymorphicUserObjectResponseBody],
        FieldMetadata(alias="assignedTo"),
        pydantic.Field(alias="assignedTo"),
    ] = None
    created_at_time: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="createdAtTime"),
        pydantic.Field(
            alias="createdAtTime", description="Creation time of the issue. UTC timestamp in RFC 3339 format."
        ),
    ]
    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Description of the issue. Included if the issue was given a description.
    """

    due_date: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="dueDate"),
        pydantic.Field(
            alias="dueDate",
            description="Due date of the issue. UTC timestamp in RFC 3339 format. Included if the issue was assigned a due date.",
        ),
    ] = None
    external_ids: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, str]],
        FieldMetadata(alias="externalIds"),
        pydantic.Field(alias="externalIds", description="A map of external ids"),
    ] = None
    id: str = pydantic.Field()
    """
    ID of the issue.
    """

    issue_source: typing_extensions.Annotated[
        IssueSourceObjectResponseBody, FieldMetadata(alias="issueSource"), pydantic.Field(alias="issueSource")
    ]
    media_list: typing_extensions.Annotated[
        typing.Optional[typing.List[FormsMediaRecordObjectResponseBody]],
        FieldMetadata(alias="mediaList"),
        pydantic.Field(
            alias="mediaList", description="List of media objects for the issue. Included if the issue has media."
        ),
    ] = None
    priority: typing.Optional[IssueResponseObjectResponseBodyPriority] = pydantic.Field(default=None)
    """
    Priority of the issue. Included if the issue was assigned a priority.  Valid values: `low`, `medium`, `high`
    """

    status: IssueResponseObjectResponseBodyStatus = pydantic.Field()
    """
    Status of the issue.  Valid values: `open`, `inProgress`, `resolved`, `dismissed`
    """

    submitted_at_time: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="submittedAtTime"),
        pydantic.Field(
            alias="submittedAtTime", description="Submission time of the issue. UTC timestamp in RFC 3339 format."
        ),
    ]
    submitted_by: typing_extensions.Annotated[
        FormsPolymorphicUserObjectResponseBody, FieldMetadata(alias="submittedBy"), pydantic.Field(alias="submittedBy")
    ]
    title: str = pydantic.Field()
    """
    Title of the issue.
    """

    updated_at_time: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="updatedAtTime"),
        pydantic.Field(
            alias="updatedAtTime", description="Update time of the issue. UTC timestamp in RFC 3339 format."
        ),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .tachograph_activity_list_wrapper import TachographActivityListWrapper

DriverTachographActivityData = typing.List[TachographActivityListWrapper]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .v_1_safety_report_harsh_event import V1SafetyReportHarshEvent


class V1DriverSafetyScoreResponse(UniversalBaseModel):
    """
    Safety score details for a driver
    """

    crash_count: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="crashCount"),
        pydantic.Field(alias="crashCount", description="Crash event count"),
    ] = None
    driver_id: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="driverId"), pydantic.Field(alias="driverId", description="Driver ID")
    ] = None
    harsh_accel_count: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="harshAccelCount"),
        pydantic.Field(alias="harshAccelCount", description="Harsh acceleration event count"),
    ] = None
    harsh_braking_count: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="harshBrakingCount"),
        pydantic.Field(alias="harshBrakingCount", description="Harsh braking event count"),
    ] = None
    harsh_events: typing_extensions.Annotated[
        typing.Optional[typing.List[V1SafetyReportHarshEvent]],
        FieldMetadata(alias="harshEvents"),
        pydantic.Field(alias="harshEvents"),
    ] = None
    harsh_turning_count: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="harshTurningCount"),
        pydantic.Field(alias="harshTurningCount", description="Harsh turning event count"),
    ] = None
    safety_score: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="safetyScore"),
        pydantic.Field(
            alias="safetyScore",
            description="The driver’s Safety Score for the requested period. Note that if the driver has zero drive time in this period, the Safety Score will be returned as 100.",
        ),
    ] = None
    safety_score_rank: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="safetyScoreRank"),
        pydantic.Field(alias="safetyScoreRank", description="Safety Score Rank"),
    ] = None
    time_over_speed_limit_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="timeOverSpeedLimitMs"),
        pydantic.Field(
            alias="timeOverSpeedLimitMs", description="Amount of time driven over the speed limit in milliseconds"
        ),
    ] = None
    total_distance_driven_meters: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="totalDistanceDrivenMeters"),
        pydantic.Field(alias="totalDistanceDrivenMeters", description="Total distance driven in meters"),
    ] = None
    total_harsh_event_count: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="totalHarshEventCount"),
        pydantic.Field(alias="totalHarshEventCount", description="Total harsh event count"),
    ] = None
    total_time_driven_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="totalTimeDrivenMs"),
        pydantic.Field(alias="totalTimeDrivenMs", description="Amount of time driven in milliseconds"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .forward_collision_detection_alert_settings_object_response_body_sensitivity import (
    ForwardCollisionDetectionAlertSettingsObjectResponseBodySensitivity,
)


class ForwardCollisionDetectionAlertSettingsObjectResponseBody(UniversalBaseModel):
    """
    Enables AI detection of near forward collisions, surfaces events in Safety Inbox, and enables configurable alerts. While the feature is in beta, it is only enabled during daytime driving hours. In-cab alerts are recommended for testing use only.
    """

    has_in_cab_audio_alerts_enabled: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="hasInCabAudioAlertsEnabled"),
        pydantic.Field(
            alias="hasInCabAudioAlertsEnabled",
            description="Indicates whether in-cab audio alerts for forward collision are turned on.",
        ),
    ] = None
    is_enabled: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="isEnabled"),
        pydantic.Field(
            alias="isEnabled", description="Indicates whether AI event detection for forward collision is turned on."
        ),
    ] = None
    sensitivity: typing.Optional[ForwardCollisionDetectionAlertSettingsObjectResponseBodySensitivity] = pydantic.Field(
        default=None
    )
    """
    Ranges from near forward collision (results in fewer events) to far forward collision (results in more events).  Valid values: `near`, `medium`, `far`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

DeviceResponseResponseBodyModel = typing.Union[
    typing.Literal[
        "AG24",
        "AG24EU",
        "AG26",
        "AG26EU",
        "AG45",
        "AG45EU",
        "AG46",
        "AG46EU",
        "AG46P",
        "AG46PEU",
        "AG51",
        "AG51EU",
        "AG52",
        "AG52EU",
        "AG53",
        "AG53EU",
        "AT11",
        "AT11X",
        "CM31",
        "CM32",
        "CM33",
        "CM34",
        "OEM",
        "OEMP",
        "OEMR",
        "OEMV",
        "VG34",
        "VG34EU",
        "VG34FN",
        "VG34M",
        "VG54EU",
        "VG54NA",
        "VG55EU",
        "VG55NA",
    ],
    typing.Any,
]
